/*
 Copyright (c) 2013 yvt

 This file is part of OpenSpades.

 OpenSpades is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenSpades is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenSpades.  If not, see <http://www.gnu.org/licenses/>.

 */

namespace spades {
	class ThirdPersonRifleSkin : IToolSkin, IThirdPersonToolSkin, IWeaponSkin, IWeaponSkin2,  IWeaponSkin3 {
		private float sprintState;
		private float raiseState;
		private Vector3 teamColor;
		private bool muted = true;
		private Matrix4 originMatrix;
		private float aimDownSightState;
		private float readyState;
		private bool reloading;
		private float reloadProgress;
		private int ammo, clipSize;

		private float environmentRoom;
		private float environmentSize;
		private float environmentDistance;
		private Vector3 soundOrigin;

		float SprintState { set { sprintState = value; } }
		float RaiseState { set { raiseState = value; } }
		bool IsMuted { set { muted = value; } }
		Vector3 TeamColor { set { teamColor = value; } }
		Matrix4 OriginMatrix { set { originMatrix = value; } }
		float PitchBias { get { return 0.0F; } }
		float AimDownSightState { set { aimDownSightState = value; } }

		bool IsReloading { set { reloading = value; } }
		float ReloadProgress { set { reloadProgress = value; } }
		int Ammo { set { ammo = value; } }
		int ClipSize { set { clipSize = value; } }
		float ReadyState { set { readyState = value; } }

		// IWeaponSkin2
		void SetSoundEnvironment(float room, float size, float distance) {
			environmentRoom = room;
			environmentSize = size;
			environmentDistance = distance;
		}
		Vector3 SoundOrigin { set { soundOrigin = value; } }

		// IWeaponSkin3
		Vector3 MuzzlePosition { get { return originMatrix * Vector3(0.35F, -1.85F, -0.125F); } }
		Vector3 CaseEjectPosition { get { return originMatrix * Vector3(0.35F, -0.7F, -0.125F); } }

		private Renderer@ renderer;
		private AudioDevice@ audioDevice;
		private AudioChunk@ fire;
		private AudioChunk@ fireDistant;
		private AudioChunk@ fire_S;
		private AudioChunk@ fireDistant_S;
		private AudioChunk@ reloadSound;

		private Model@ t0;
		private Model@ t1;
		private Model@ t2;
		private Model@ t3;
		private Model@ t4;
		private Model@ t5;
		private Model@ t6;

		private ConfigItem sniper_supp("sniper_supp", "0");

		ThirdPersonRifleSkin(Renderer@ r, AudioDevice@ dev) {
			@renderer = r;
			@audioDevice = dev;
			@t0 = renderer.RegisterModel("Models/Weapons/Rifle/t0.kv6");
			@t1 = renderer.RegisterModel("Models/Weapons/Rifle/t1.kv6");
			@t2 = renderer.RegisterModel("Models/Weapons/Rifle/t2.kv6");
			@t3 = renderer.RegisterModel("Models/Weapons/Rifle/t3.kv6");
			@t4 = renderer.RegisterModel("Models/Weapons/Rifle/t4.kv6");
			@t5 = renderer.RegisterModel("Models/Weapons/Rifle/t5.kv6");
			@t6 = renderer.RegisterModel("Models/Weapons/Rifle/t6.kv6");

			@fire = dev.RegisterSound("Sounds/Weapons/Rifle/fire.wav");
			@fireDistant = dev.RegisterSound("Sounds/Weapons/Rifle/fire_distant.wav");
			@fire_S = dev.RegisterSound("Sounds/Weapons/Rifle/suppressed.wav");
			@fireDistant_S = dev.RegisterSound("Sounds/Weapons/Rifle/suppressed_distant.wav");
			@reloadSound = dev.RegisterSound("Sounds/Weapons/Rifle/Reload.opus");
		}

		void Update(float dt) {}

		void WeaponFired() {
			if (!muted) {
				if (sniper_supp.IntValue > 0) {
					Vector3 origin = soundOrigin;
					AudioParam param;
					param.volume = 20.0F;
					audioDevice.Play(fire_S, origin, param);

					param.volume = 1.0F;
					param.referenceDistance = 26.0F;
					audioDevice.Play(fireDistant_S, origin, param);

					param.referenceDistance = 4.0F;
					param.volume = 1.0F;
					audioDevice.Play(fire_S, origin, param);
				} else {
					Vector3 origin = soundOrigin;
					AudioParam param;
					param.volume = 20.0F;
					audioDevice.Play(fire, origin, param);

					param.volume = 1.0F;
					param.referenceDistance = 26.0F;
					audioDevice.Play(fireDistant, origin, param);

					param.referenceDistance = 4.0F;
					param.volume = 1.0F;
					audioDevice.Play(fire, origin, param);
				}
			}
		}

		void ReloadingWeapon() {
			if (!muted) {
				Vector3 origin = soundOrigin;
				AudioParam param;
				param.volume = 0.2F;
				audioDevice.Play(reloadSound, origin, param);
			}
		}

		void ReloadedWeapon() {}

		Matrix4 CreateEulerAnglesMatrix(Vector3 angles) {
			Matrix4 mat = CreateRotateMatrix(Vector3(1, 0, 0), angles.x);
			mat = CreateRotateMatrix(Vector3(0, 1, 0), angles.y) * mat;
			mat = CreateRotateMatrix(Vector3(0, 0, 1), angles.z) * mat;
			return mat;
		}

		void AddToScene() {
			
			Matrix4 mat = CreateScaleMatrix(0.05F);
			mat = mat * CreateScaleMatrix(-1, -1, 1);
			
			Vector3 trans = Vector3(0.4F, -0.9F, 0.0F);
			trans -= 0.01F; // stop z-fighting
			mat = CreateTranslateMatrix(trans) * mat;

			ModelRenderParam param;
			param.matrix = originMatrix
			* CreateScaleMatrix(0.003)
			* CreateTranslateMatrix(100, -150, 100);
			renderer.AddModel(t0, param);
			renderer.AddModel(t1, param);
			renderer.AddModel(t2, param);
			renderer.AddModel(t3, param);
			renderer.AddModel(t4, param);
			if (sniper_supp.IntValue == 1) {
				renderer.AddModel(t6, param);
			} else {
				renderer.AddModel(t5, param);
			}
		}
	}

	IWeaponSkin@ CreateThirdPersonRifleSkin(Renderer@ r, AudioDevice@ dev) {
		return ThirdPersonRifleSkin(r, dev);
	}
}